#!/bin/env python


# (c) 2000, Free Software Foundation
#
# Gilgamesh is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Gilgamesh is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iMailer; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
import sys, os, os.path, time

import soya
import soya.model
import soya.soya3d as soya3d

import  soya.widget as widget, soya.idler as idler

from baseObjets import Cylinder, Sphere, Line

class carriere:
  """ contient diffrents matriaux """
  
  def __init__(self):

    # materiau par dfaut
    self.defaut = soya.model.Material()

    # materiau des torpilles
    self.torp = soya.model.Material()
    self.torp.diffuse   = (1.0, 0.0, 0.0, 1.0)
    self.torp.shininess = 128.0

    # materiau des subs
    self.perso = soya.model.Material()
    self.perso.diffuse   = (0.0, 0.0, 1.0, 1.0)
    self.perso.shininess = 128.0
    
    # materiau des ennemis
    self.ennemi = soya.model.Material()
    self.ennemi.diffuse   = (0.5, 0.0, 0.0, 1.0)
    self.ennemi.shininess = 128.0
    
    # materiau des amis
    self.ami = soya.model.Material()
    self.ami.diffuse   = (0.0, 0.5, 0.0, 1.0)
    self.ami.shininess = 128.0

    # materiau de la grille lache
    self.grille_lache = soya.model.Material()
    self.grille_lache.diffuse   = (1.0, 1.0, 1.0, 0.7)

    # materiau de la grille fine
    self.grille_fine = soya.model.Material()
    self.grille_fine.diffuse   = (1.0, 1.0, 1.0, 0.3)

    ## matriaux
    # .shininess( value )
    # Sets the material's shininess to 0.5. The shininess ranges from 0.0 to 128.0;
    # 0.0 is the most metallic / shiny, and 128.0 is the most plastic.

    # .diffuse( value )
    # Sets the material's diffuse color. The diffuse color is the basic color.
    # In Soya 3D, colors are tupples of 4 floats: (red, green, blue, alpha), where
    # each component normally ranges from 0.0 to 1.0 (though you can use values
    # out of this range, e.g. to get a dark light), and alpha is used for
    # transparency (1.0 is opaque and 0.0 fully transparent).

    # .specular( r,g,b,a )
    # Sets the material's specular color. The specular color is the one used for
    # the specular / shiny effects.

    # Other interesting attributes of materials are:
    # - wireframed        : draw in wireframe
    # - additive_blending : usefull for alpha-blending and special effect
    # - tex_name          : 
    # - separate_specular : 
    

class zone:
  """ la zone contient tout les lements statiques : dcors, atmosphere, lumires """
  def __init__(self, scene):

    # paramtres rglables
    self.largeur = 20.0
    self.longueur = 20.0
    self.hauteur = 0.0

    # variables internes
    self.taille_x = self.largeur
    self.taille_y = self.hauteur
    self.taille_z = self.longueur
    
    self.centre_x = self.largeur / 2
    self.centre_y = self.hauteur / 2
    self.centre_z = self.longueur / 2

    # materiaux
    self.materiaux = carriere()

    # dcors supplmentaires insr dans la scene gnrale
    self.decors = soya3d.World( parent = scene, name = "decors" )
    
    # atmosphere
    self.atmosphere = soya3d.Atmosphere()
    scene.atmosphere = self.atmosphere
    
    
  def ajouteGrilleCarre(self, taille=20.0, maille=4.0, materiau=None, y=0.0, nom = "" ):
    # si pas de matriau donn, par dfaut
    if materiau==None:
      materiau = self.materiaux.defaut

    # insere l'ensemble dans le dcors
    grille = soya3d.World( parent = self.decors, name = nom )
    
    i=0
    for j in range( maille+1 ):
      
      # ligne verticale
      ligneV = Line(grille, materiau, length=taille ) 
      ligneV.set_xyz( i, y, 0 )
      ligneV.rotate_lateral( 0.0 )
      
      # ligne horizontale
      ligneH = Line(grille, materiau, length=taille )
      ligneH.set_xyz( taille, y, i )
      ligneH.rotate_lateral( -90.0 )
      
      i += taille/maille


      
  def ajouteGrilleCarre_Lache(self):
    self.ajouteGrilleCarre( taille = self.taille_x, maille=4, materiau=self.materiaux.grille_lache, y=-0.1, nom="grille_lache" )

  def ajouteGrilleCarre_Fine(self):
    self.ajouteGrilleCarre( taille = self.taille_x, maille=20, materiau=self.materiaux.grille_fine, y=-0.1, nom="grille_fine" )


  def ajouteLumiere(self, x=0.0, y= 20.0, z=0.0, nom = ""):
    # ajoute une lumire dans le dcors
    lumiere = soya.soya3d.Light( self.decors, name=nom )
    lumiere.set_xyz( x,y,z )

  def ajouteLumiere_Centrale(self):
    # ajoute une lumire au centre de la scne
    self.ajouteLumiere( self.centre_x, 10.0, self.centre_z, nom="lumiere_centrale" )
    

  def ajouteAtmosphere(self):
    
    # l'ambiance de la scene
    
    # Lumire ambiante ( rouge, vert, bleu, transparence )
    self.atmosphere.ambient = (0.3, 0.3, 0.5, 1.0)
    
    # Couleur de fond
    self.atmosphere.bg_color = (0.0, 0.0, 0.2, 1.0)
    
    # Brouillard
    self.atmosphere.fog_color = (0.0, 0.0, 0.2, 1.0)
    
    # Types de brouillards
    #  * 0 = linear      (GL_LINEAR)
    #  * 1 = exponential (GL_EXP)
    #  * 2 = exponential (GL_EXP2)
    self.atmosphere.fog_type    = 2
    self.atmosphere.fog_start   = 0.0
    self.atmosphere.fog_end     = 35.0
    self.atmosphere.fog_density = 0.05
    
    # Brouillard disponible ou non : 0 = non, 1 = oui
    self.atmosphere.fog = 0
    
    # Cette self.atmospherephere est celle de base, on lui attribue un ciel
    self.atmosphere.skyplane  = 1
    
    # Ciel disponible ou non : 0 = non, 1 = oui
    self.atmosphere.sky_color = (0.5, 0.5, 1.0, 1.0)
    
    # Le ciel consiste en un gradient allant de self.atmospherephere.sky_color (haut) 
    # self.atmospherephere.bg_color (bas). On peut ajouter des nuages avec des materiaux
    soya.model.Image.PATH = os.path.join(os.path.dirname(os.path.abspath(sys.argv[0])), "data")
    self.atmosphere.cloud = soya.model.Material()
    self.atmosphere.cloud.tex_filename = "cloud_2.tga"
    


class traceur:
  def __init__(self):

    # initialise soya
    soya.init()
    
    # Cr une scene
    self.scene = soya3d.World()
    
    # cr une zone dans la scene de base
    self.zone = zone( self.scene )

    # y ajoute diffrents lements
    self.zone.ajouteAtmosphere()
    self.zone.ajouteLumiere_Centrale()
    self.zone.ajouteGrilleCarre_Lache()
    self.zone.ajouteGrilleCarre_Fine()

    # cr un conteneur pour les objets
    self.objets = soya3d.World( self.scene, name = "objets" )

    # groupe contenant les tiquettes de texte
    self.labels = soya.widget.Group()
    
    # Met un texte descriptif pour la zone
    nomPartie = "Clan Argonaute"
    numTour = 5
    text = soya.widget.Label(self.labels, text= nomPartie+" - tour "+str(numTour), color1=[1.0,1.0,1.0,0.7] )
    
    text.top=0
    text.left=0
    text.width = 200
    text.height=200
    text.scale=(0.7,0.7)


    # type de modle pour les vecteurs de vitesse des objets
    # 0 = ligne, 1 = cylindre
    self.vecteurType = 1

    # widget de base
    soya.set_root_widget(widget.Group())
    
    # objet contenant tout les matriaux
    self.materiaux = carriere()
    
    # ajoute une camra par dfaut
    self.vueDessus()

    # FIXME : variable temporaire pour positionnement tiquettes    
    self.curpos = 0

    
  def ajouteObjetVecteur(self, x = 10.0, y = 0.0, z = 10.0, cap = 0.0, vitesse = 0.0, materiau=None, nom="", classe="", id = "" ):
    if materiau==None:
      materiau=self.mat_perso

    # Met un texte descriptif pour l'objet dans le groupe labels
    label = str(nom)+"\n"+str(classe)+" ("+str(int(vitesse))+"n,"+str(int(cap))+","+str(int(x))+","+str(int(z))+")"
    etiquette = soya.widget.Label(self.labels, text=label, color1=[1.0,1.0,1.0,0.5], color2=materiau.diffuse )

    # FIXME : mettre les tiquettes  cot des subs?
    #h = soya.get_screen_height()
    #w = soya.get_screen_width()
    
    self.curpos += 1
    etiquette.top = self.curpos * 40
    etiquette.left = 0
    etiquette.width = 200
    etiquette.height = 200
    etiquette.scale = (0.5,0.5)
    
    # conversion en coordonnes esubiennes
    z = -1*(z-20.0)
    cap = -1*cap+180
    # conversion en noeuds
    vitesse = vitesse / 10


    # on insre ce nouvel objet dans les objets
    objetVecteur = soya3d.World( self.objets, name = id )
    
      
    # ajoute une boule  la position donne
    # la boule  le diamtre de la sensibilit des torpille / 2
    objetPos = Sphere(objetVecteur, materiau, radius=0.1 )
    objetPos.set_xyz( x, y, z )

    # boule transparente de diamtre de la sensibilit des torpilles
    ## mat_temp = soya.model.Material()
    ## mat_temp.diffuse = (1.0,1.0,1.0, 0.5)
    ## mat_temp.shininess = 128.0
    ## self.sphere = Sphere(self.scene, mat_temp, radius=0.2)
    ## self.sphere.set_xyz( x, y, z )

    
    # ajoute une lumire au dessus de la position de l'objet
    #self.light = soya.soya3d.Light(self.scene)
    #self.light.set_xyz( x, y + 10, z)
    
    # ajoute le vecteur vitesse
    # choix du modle pour le vecteur vitesse
    if self.vecteurType==0:
      # cr une ligne
      objetDir = Line(objetVecteur, materiau, length=vitesse)
      # change les coordonnes
      objetDir.set_xyz( x,y, z )
      # change l'angle
      objetDir.rotate_lateral( cap )

    else:
      # cr un cylindre
      objetDir = Cylinder(objetVecteur, materiau, length=vitesse, radius=0.03)
      objetDir.set_xyz( x, y, z )
      objetDir.rotate_lateral( cap )


  def ajouteSubPerso(self, x = 10.0, z = 10.0, cap = 0.0, vitesse = 0.0, nom="", classe="", id = "" ):
    self.ajouteObjetVecteur( x=x, z=z, cap=cap, vitesse=vitesse, materiau=self.materiaux.perso, nom=nom, classe=classe, id=id )
    
  def ajouteSubAmi(self, x = 10.0, z = 10.0, cap = 0.0, vitesse = 0.0, classe="", id = "" ):
    self.ajouteObjetVecteur( x=x, z=z, cap=cap, vitesse=vitesse, materiau=self.materiaux.ami, classe=classe, id=id )

  def ajouteSubEnnemi(self, x = 10.0, z = 10.0, cap = 0.0, vitesse = 0.0, classe="", id = "" ):
    self.ajouteObjetVecteur( x=x, z=z, cap=cap, vitesse=vitesse, materiau=self.materiaux.ennemi, classe=classe, id=id )
    
  def ajouteTorp(self, x = 10.0, z = 10.0, cap = 0.0, vitesse = 0.0, classe="", id = "" ):
    self.ajouteObjetVecteur( x=x, z=z, cap=cap, vitesse=vitesse, materiau=self.materiaux.torp, classe=classe, id=id  )
    

  def positionneCamera(self, x=0.0, y=20.0, z=0.0, cible_x=10.0, cible_y=0.0, cible_z=10.0):
    # cr une camra
    self.camera = soya3d.Camera(self.scene)
    # position
    self.camera.set_xyz( x, y, z )
    # rotation pour tre dans la bonne direction
    #self.camera.rotate_lateral(180.0)
    # direction o la camra regarde
    self.camera.look_at(soya3d.Point(self.scene, cible_x, cible_y, cible_z ))

  def vueDessus(self):
    self.positionneCamera( self.zone.centre_x, 20.0, self.zone.centre_z )
    
  def vueTroisQuart(self):
    self.positionneCamera( self.zone.centre_x, 7.0, self.zone.taille_z + 12.0 )
    
  def vuePerspective(self):
    self.positionneCamera( self.zone.taille_x - 24.5, 7.0, self.zone.taille_z + 4.5 )
    

  def afficher(self):
    # ajoute la camra
    soya.root_widget.add(self.camera)
    # ajoute les tiquettes
    soya.root_widget.add(self.labels)
    
    #for child in self.objets.subitem("1") : print child
    
    # et boucle sur l'affichage
    #idler.Idler(self.scene).idle()
    from math import cos, sin
    
    
    pi = 3.141592
    i= pi / 2
    j=0.0
    while i<2*pi:
      soya.render()
      
      j=j+0.5
      self.objets.subitem("1").set_xyz( j, 0.0, -j )
      
      i = i+ pi / 20
      self.camera.set_xyz( 20.0*cos(i) +10.0 , self.camera.y, 20.0*sin(i) + 10.0 )
      self.camera.look_at(soya3d.Point(self.scene, 10.0, 0.0, 10.0 ))

      #time.sleep(0.0001)
    
    """
    import soya.game.camera as game_camera
    self.cam = game_camera.ThirdPersonTraveling( target = soya3d.Point(self.objets.subitem("1"), 0.0, 0.0, 0.0 ))#, direction = soya3d.Point(1.0,0.0,1.0) )
    soya.root_widget.add(self.cam)
    
    i=0.0
    while i<1.0:
      soya.render()
      i=i+0.1
    """  

